/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.util.ListIterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author stefan
 */
public class Presentation {
    int aspect_x, aspect_y;
    Vector slides;

    Presentation (int aspectX, int aspectY)
    {
	aspect_x = aspectX;
	aspect_y = aspectY;
	slides = new Vector();
    }

    Slide addSlide()
    {
	Slide slide = new Slide (this);
	slides.add (slide);
	return slide;
    }

    void loadPresentation (XMLStreamReader r) throws XMLStreamException
    {
	for (int i = 0; i < r.getAttributeCount(); i++)
        {
            String attr_name = r.getAttributeLocalName (i);
            String attr_value = r.getAttributeValue (i);
            if (attr_name.equals ("aspect_x"))
                aspect_x = Integer.parseInt (attr_value);
            if (attr_name.equals ("aspect_y"))
                aspect_y = Integer.parseInt (attr_value);
        }
	while (r.hasNext())
	{
	    int event = r.next();
	    switch (event)
	    {
		case XMLStreamConstants.START_ELEMENT:
		    if (r.getLocalName().equals ("slide"))
		    {
			Slide slide = addSlide();
			slide.load (r);
		    }
		case XMLStreamConstants.END_ELEMENT:
		    if (r.getLocalName().equals("presentation"))
			return;
	    }
	}
    }
    boolean loadZipEntry (File file, int n)
    {
        ZipInputStream zip_in_stream = null;
        try {
            zip_in_stream = new ZipInputStream (new FileInputStream (file));
            ZipEntry entry;
            entry = zip_in_stream.getNextEntry();
            while (n-- > 0)
                entry = zip_in_stream.getNextEntry();
            if (entry == null)
                return false;
            if (entry.getName().equals ("cut_and_play_slides.xml"))
            {
		XMLInputFactory f = XMLInputFactory.newInstance();
                XMLStreamReader r = f.createXMLStreamReader(zip_in_stream);
                while (r.hasNext())
                {
                    int event = r.next();
                    switch (event)
                    {
                        case XMLStreamConstants.START_ELEMENT:
                            {
                                if (r.getLocalName().equals ("presentation"))
				{
				    loadPresentation (r);
				}
                            }
                           break;
                        case XMLStreamConstants.END_ELEMENT:
                        // r.getLocalName();
                    }
                }
                r.close();
            }
       	    ListIterator iter = slides.listIterator();
	    while (iter.hasNext())
	    {
		Slide slide = (Slide) iter.next();
		if (slide.backgroundImage != null &&
		    slide.backgroundImage.fileName != null &&
		    entry.getName().equals (slide.backgroundImage.fileName))
		{
		    slide.backgroundImage.loadFromZip (zip_in_stream);
		}
	    }
	    return true;
        }
        catch (XMLStreamException ex) {
            Logger.getLogger(Slide.class.getName()).log(Level.SEVERE, null, ex);
        }
        catch (IOException ex) {
                Logger.getLogger(Slide.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                zip_in_stream.close();
            } catch (IOException ex) {
                Logger.getLogger(Slide.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return false;
    }

    boolean load (File file)
    {
        /* reset presentation contents */
        slides.clear();

        /* load presentation contents */
        int n = 0;
        while (loadZipEntry (file, n))
            n++;

	if (slides.size() > 0)
	    return true;
	else
	    return false;
    }

    boolean save (File file) {
	boolean success = true;
        ZipOutputStream out = null;
        try {
            out = new ZipOutputStream (new FileOutputStream (file));
            out.putNextEntry (new ZipEntry ("cut_and_play_slides.xml"));

            OutputStreamWriter writer = null;
            writer = new OutputStreamWriter (out, Charset.forName("utf-8"));

            try {
                XMLOutputFactory xof = XMLOutputFactory.newInstance();
                XMLStreamWriter xtw = null;
                xtw = xof.createXMLStreamWriter(writer);
                xtw.writeStartDocument("utf-8", "1.0");
                xtw.writeCharacters("\n");
                xtw.writeComment("Cut-and-Play Presentation Format");
                xtw.writeCharacters("\n");
                xtw.writeStartElement("presentation");
		xtw.writeAttribute ("aspect_x", Integer.toString (aspect_x));
		xtw.writeAttribute ("aspect_y", Integer.toString (aspect_y));
                xtw.writeCharacters ("\n");
                ListIterator iter = slides.listIterator();
                while (iter.hasNext()) {
                    Slide slide = (Slide) iter.next();
                    slide.save(xtw);
                    xtw.writeCharacters("\n");
                }
                xtw.writeEndElement();
                xtw.writeCharacters("\n");
                xtw.writeEndDocument();
                xtw.flush();
                xtw.close();
            } catch (XMLStreamException ex) {
		// Logger.getLogger(Presentation.class.getName()).log(Level.SEVERE, null, ex);
		success = false;
            }
            out.closeEntry();
	    /* FIXME: md5 file names */
	    ListIterator iter = slides.listIterator();
	    while (iter.hasNext())
	    {
		Slide slide = (Slide) iter.next();
		if (slide.backgroundImage != null)
		{
		    String file_name = slide.backgroundImage.fileName;
		    out.putNextEntry (new ZipEntry (file_name));
		    InputStream in = new ByteArrayInputStream (slide.backgroundImage.imageData);
		    byte data[] = new byte[1024];
		    int read;
		    while ((read = in.read (data)) > 0)
			out.write(data, 0, read);
		    out.closeEntry();
		}
	    }
            out.close();
        } catch (IOException ex) {
	    // Logger.getLogger(Presentation.class.getName()).log(Level.SEVERE, null, ex);
	    success = false;
        }
	return success;
    }
}
