/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author stefan
 */
abstract class SlideElement {
    boolean selected;

    SlideElement()
    {
        selected = false;
    }
    abstract boolean at (Point p, Rectangle r);
    abstract void mousePressed (Point p, Rectangle r);
    abstract void mouseDragged (Point p, Rectangle r);
    abstract void mouseReleased (Point p, Rectangle r);
    abstract void keyTyped (KeyEvent e);
    abstract void draw (Graphics2D g2, Rectangle rectangle);
    abstract void save (XMLStreamWriter xtw) throws XMLStreamException;
    abstract void load (XMLStreamReader r) throws XMLStreamException;
    abstract void showToolPanel (ToolPanelProvider provider);
    abstract int layer();
    void setSelected (boolean newSelected)
    {
        selected = newSelected;
    }
    void writeAttributeColor (XMLStreamWriter xtw, String name, Color color) throws XMLStreamException
    {
        String color_string = new String ("#");

        String string = new String();
        string = Integer.toHexString (color.getRed());
        while (string.length() < 2)
          string = "0" + string;
        color_string += string;
        string = Integer.toHexString (color.getGreen());
        while (string.length() < 2)
          string = "0" + string;
        color_string += string;
        string = Integer.toHexString (color.getBlue());
        while (string.length() < 2)
          string = "0" + string;
        color_string += string;
        xtw.writeAttribute (name, color_string);
    }

    Color parseAttributeColor (String attr_value)
    {
        String color_r = attr_value.substring (1, 3);
        String color_g = attr_value.substring (3, 5);
        String color_b = attr_value.substring (5, 7);
        return new Color (Integer.parseInt (color_r, 16),
                          Integer.parseInt (color_g, 16),
                          Integer.parseInt (color_b, 16));
    }
}
