/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author stefan
 */
public class Text extends SlideElement {
    String text;
    Rectangle space;
    Dimension virtual_screen;
    Slide slide;

    Point clickPosition; // for dragging
    Point spacePosition; // for dragging
    private Color color;
    private int fontSize;
    Text (Slide slide)
    {
        color = Color.WHITE;
        this.slide = slide;
        text = new String ("Text");
        virtual_screen = new Dimension (3000 * slide.presentation.aspect_x / slide.presentation.aspect_y, 3000);
        space = new Rectangle();
        space.x = 250;
        space.y = 250;
        space.width = 10;
        space.height = 300;
        fontSize = 70 * 4;
    }
    public void keyTyped (KeyEvent e)
    {
        char key = e.getKeyChar();
        if (key == KeyEvent.VK_ENTER)
        {
            text = text + "\n";
        }
        if (key == KeyEvent.VK_BACK_SPACE)
        {
            if (text.length() > 0)
                text = text.substring (0, text.length() - 1);
        }
        else if (key >= KeyEvent.VK_SPACE)
        {
            text += key;
        }
        slide.view.redraw();
    }
    public void draw (Graphics2D g2, Rectangle r)
    {
        String drawText = text;
        BufferedImage bi;
        bi = new BufferedImage (space.width, space.height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g3 = bi.createGraphics();
        Font f = new Font ("Dialog", Font.PLAIN, fontSize);
        g3.setColor (color);
        g3.setFont (f);
        FontMetrics fm = g3.getFontMetrics (g3.getFont());

	int lineHeight = fm.getHeight();
        
        if (slide.view.focusElement == this)
            drawText = text + "_";

        String[] lines = drawText.split("\n");
        /* check height of offscreen image */
        if (space.height < lines.length * lineHeight)
        {
	    space.height = lines.length * lineHeight;
	    draw (g2, r);
	    return;
	}
        /* check width of offscreen image */
        for (int i = 0; i < lines.length; i++)
        {
            int width = fm.stringWidth (lines[i]);
            if (width > space.width)
            {
                space.width = width;
                draw (g2, r);
                return;
            }
        }
        for (int i = 0; i < lines.length; i++)
        {
            g3.drawString (lines[i], 0, i * lineHeight + fm.getAscent());
        }

        AffineTransform t = new AffineTransform();
        double xpos = (double) (space.x * r.width) / virtual_screen.width;
        double ypos = (double) (space.y * r.height) / virtual_screen.height;
        t.translate (r.x + xpos, r.y + ypos);
        t.scale ((double) r.height / virtual_screen.height, (double) r.width / virtual_screen.width);
        g2.drawImage (bi, t, null);
        if (slide.view.focusElement == this)
        {
            float dash1[] = { 10.0f };
            BasicStroke dashed = new BasicStroke(1.0f,
                         BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);
            Rectangle frame = new Rectangle (r.x + (int)xpos, r.y + (int)ypos, space.width * r.width / virtual_screen.width, space.height * r.height / virtual_screen.height);
            g2.setColor(Color.BLACK);
            g2.draw(frame);
            g2.setColor (Color.YELLOW);
            g2.setStroke (dashed);
            g2.draw(frame);
        }
    }
    void mousePressed (Point p, Rectangle r)
    {
        clickPosition = p;
        spacePosition = new Point (space.x, space.y);
        slide.view.redraw();
    }
    void mouseDragged (Point p, Rectangle r)
    {
        double vx = (double) (p.x - clickPosition.x) / r.width * virtual_screen.width;
        double vy = (double) (p.y - clickPosition.y) / r.height * virtual_screen.height;
        space.x = spacePosition.x + (int) vx;
        space.y = spacePosition.y + (int) vy;
        if (space.x < 0) {
            space.x = 0;
        }
        if (space.y < 0) {
            space.y = 0;
        }
        int x_outside = space.x + space.width - virtual_screen.width;
        if (x_outside > 0) {
            space.x -= x_outside;
        }
        int y_outside = space.y + space.height - virtual_screen.height;
        if (y_outside > 0) {
            space.y -= y_outside;
        }
        slide.view.redraw();
    }
    void mouseReleased (Point p, Rectangle r)
    {

    }
    @Override
    boolean at(Point p, Rectangle r) {
        double vx = (double) (p.x - r.x) / r.width * virtual_screen.width;
        double vy = (double) (p.y - r.y) / r.height * virtual_screen.height;
        if (space.contains (new Point ((int)vx, (int)vy)))
            return true;
        return false;
    }

    void setFontSize(int i) {
        fontSize = i;
        space.height = space.width = 10;  // will be recalculated
        slide.view.redraw();
    }
    int getFontSize()
    {
        return fontSize;
    }

    @Override
    void showToolPanel(ToolPanelProvider provider) {
        provider.showTextPanel (this);
    }

    Color getColor() {
        return color;
    }

    void setColor (Color newColor)
    {
        color = newColor;
        slide.view.redraw();
    }

    @Override
    int layer() {
        return 2;
    }

    @Override
    void save(XMLStreamWriter xtw) throws XMLStreamException {
        xtw.writeStartElement("text");
        xtw.writeAttribute("x", Integer.toString (space.x));
        xtw.writeAttribute("y", Integer.toString (space.y));
        xtw.writeAttribute("font_size", Integer.toString (fontSize));
        writeAttributeColor(xtw, "color", color);
        xtw.writeCharacters(text);
        xtw.writeEndElement();
    }


    @Override
    void load (XMLStreamReader r) throws XMLStreamException
    {
        for (int i = 0; i < r.getAttributeCount(); i++)
        {
            String attr_name = r.getAttributeLocalName (i);
            String attr_value = r.getAttributeValue (i);
            if (attr_name.equals ("x"))
                space.x = Integer.parseInt (attr_value);
            if (attr_name.equals ("y"))
                space.y = Integer.parseInt (attr_value);
            if (attr_name.equals ("font_size"))
                fontSize = Integer.parseInt(attr_value);
            if (attr_name.equals ("color"))
                color = parseAttributeColor (attr_value);
        }
        text = "";
        while (r.hasNext())
        {
            int event = r.next();

            if (event == XMLStreamConstants.CHARACTERS)
            {
                int start = r.getTextStart();
                int length = r.getTextLength();
                text += new String (r.getTextCharacters(), start, length);
            }
            if (event == XMLStreamConstants.END_ELEMENT)
                if (r.getLocalName().equals ("text"))
                    return;
        }
    }
}
